-------------------------------------------------------------------------------
	--                                                                      
	--                        Blink Led Example Project
	--  
	-------------------------------------------------------------------------------
	--                                                                      
	-- ENTITY:         blink_led
	--
	-- FILENAME:       blink_led.vhd
	-- 
	-- ARCHITECTURE:   rtl
	-- 
	-- ENGINEER:       Christopher Reisner
	--
	-- DATE:           17. August 2016
	--
	-- VERSION:        1.0
	--
	-------------------------------------------------------------------------------
	--                                                                      
	-- DESCRIPTION:   This is a simple blink led Example for the Zybo Development
	--				  Board. The LEDs can be controlled via AXI4 Lite.
	--
	-------------------------------------------------------------------------------
	--
	-- REFERENCES:     (none)
	--
	-------------------------------------------------------------------------------
	--                                                                      
	-- PACKAGES:       std_logic_1164 (IEEE library)
	--
	-------------------------------------------------------------------------------
	--                                                                      
	-- CHANGES:        Version 1.0 - CR - 17. August 2016
	--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

entity blink_led is
  generic (
	led_width     : integer := 4;  -- Ammount of LEDs of the Dev Board
	blink_period  : integer range 0 to 10000000 := 10000000 -- Blink frequency is currently set to 10 Hz ( blink frequency = clk_period/blink_period)
	);
  port (
    clk_in      : in std_logic; -- 100 MHz Clock signal
	reset_n_in    : in std_logic; -- Asynchronous low active reset
	leds_in     : in std_logic_vector(led_width-1 downto 0); -- Control Signal
	leds_out    : out std_logic_vector(led_width-1 downto 0)-- LEDs
  );
end blink_led;

architecture rtl of blink_led is
  
  --Enable signal for the blink LEDs
  signal clk_en_s : std_logic;
  -- Counter needed in order to generate the clk_en_s signal
  signal clk_en_counter_s : integer range 0 to 10000000;
  --D-FlipFlop for the Control Inputs
  signal led_in_dff_s : std_logic_vector(led_width-1 downto 0);
  -- Sync Signal
  -- Used to synchronize all LEDs
  signal led_sync_s : std_logic;
  -- LED signal 
  -- This signal is needed because it is not possible to read from an output Port (leds_out)
  signal leds_out_s :  std_logic_vector(led_width-1 downto 0);
  
  begin
  
  --Enable Process
  -- Used to generate the 1 Hz enable signal
  enable : process(clk_in, reset_n_in)
  begin
    if (reset_n_in = '0') then
	  clk_en_s <= '0';
	  clk_en_counter_s <= 0;
    elsif (clk_in'event and clk_in = '1' ) then
	  clk_en_s <= '0';
      clk_en_counter_s <= clk_en_counter_s + 1;
	  if (clk_en_counter_s = blink_period-1) then
		clk_en_s <= '1';
		clk_en_counter_s <= 0;
	  end if; 
    end if;
  end process;
  
  -- LED Control Process
  -- Read Inputs of the LED Control signals and store them in D-FlipFlop
  switch : process(clk_in, reset_n_in)
  begin
    if (reset_n_in = '0') then
	  led_in_dff_s <= (others => '0');
    elsif (clk_in'event and clk_in = '1' ) then
	  led_in_dff_s <= leds_in;
    end if;
  end process;
  
  -- Blink Process
  -- The LEDs will blink 5 times per second
  blink : process(clk_in, reset_n_in)
  begin
    if (reset_n_in = '0') then
	  leds_out_s <= (others => '0');
	  led_sync_s <= '0';
    elsif (clk_in'event and clk_in = '1' ) then
	  if(clk_en_s = '1') then
	    for i in 0 to led_width-1 loop
		  if (led_in_dff_s(i) = '1') then
		    if led_sync_s = '1' then 
		      leds_out_s(i)<= '1';
		      led_sync_s <= '0';
		    else
		      leds_out_s(i)<= '0';
		      led_sync_s <= '1';
		    end if;   
		  else 
	        leds_out_s(i)<= '0';
		  end if;
	    end loop;
	  end if ;
    end if;
  end process;
  
  --Connect the internally used signal leds_out_s to the output port leds_out of this module
  leds_out <= leds_out_s;
  
end rtl;